import os

def repopath(arg0 = ""):
    
    relp = os.path.dirname(arg0)
    rp = os.path.abspath(relp)

    rp = os.path.join(rp, os.pardir) 

    return os.path.abspath(rp)

def mypath(file = ""):

    if not file:
        file = __file__

    fp = os.path.dirname(file)
    rp = os.path.abspath(fp)

    return rp

def version_cache():
    return os.path.join(mypath(), "version.cache")

def version():

    vp = version_cache()

    with open(vp, "r") as src:
        for l in src.read().splitlines():

            # assuming: version: 3.15.0.1444
            pl = l.split(":")
            if len(pl) == 2:
                return pl[1]

    print("WARNING: could not parse version from " + vp)

    return ""




def build(config):
    import subprocess as sp

    if "all" not in config.project and config.name not in config.project:
        print(config.name + " not selected for building, skipping...")
        return

    if config.is_built():
        print(config.name + " exists, skipping...")
        return

    print(config.name + " ----------------------------------------------")

    cmakeconfig = "cmake " + " ".join(config.cmake_args())

    print(cmakeconfig)

    # configure
    try:
        sp.run(cmakeconfig, check=True)
    except sp.CalledProcessError:
        print("[ERROR] cmake config failed - aborting...")
        exit(1)

    if config.configure:
        print("\"configure\" specified -> done")
        return

    # build release
    sp.run("cmake --build %s --config Release %s -- -m" %
           (config.builddir, " ".join(config.cmake_build_args())), check=True)

    # build debug?!
    if config.buildconfig.lower() == "debug":
        sp.run("cmake --build %s --config Debug %s -- -m" %
               (config.builddir, " ".join(config.cmake_build_args())), check=True)

    # install
    if config.install:
        sp.run("cmake --build %s --config Release --target INSTALL -- -m" %
               (config.builddir), check=True)



#!/usr/bin/env python
"""Collects all libraries used by a given executable or library.

This script searches for all libraries used by an executable or
library, which is given to the script and copies the libraries
found into the directory of the input file. The search paths are
specified in a config file. Also different configuration have
to be specified, so that this script works with a cmake build
environment.
"""
import logging

__author__ = "Stefan Fiel, Markus Diem, Florian Kleber"
__copyright__ = "Copyright 2016, nomacs - ImageLounge"
__credits__ = ["Stefan Fiel", "Markus Diem", "Florian Kleber"]
__license__ = "GPLv3"
__version__ = "0.1"
__maintainer__ = "Stefan Fiel"
__email__ = "stefan@nomacs.org"
__status__ = "Production"


OUTPUT_NAME = "DependencyCollector"

logging.basicConfig(level=logging.INFO, format=OUTPUT_NAME +
                    ' %(levelname)s %(message)s')
# ' %(asctime)s - %(levelname)s - %(message)s')

logger = logging.getLogger()


# parses the config file and stores the values into a dictionary
def parse_config_file(configfile, conftype):
    import configparser
    config = configparser.ConfigParser()

    config.read(configfile)
    if config.has_option('DependencyCollector', 'MAPPING_'+conftype.upper()):
        mapping = config['DependencyCollector']['MAPPING_'+conftype.upper()]
    else:
        logger.error("mapping not found in the configuration file (MAPPING_" +
                     conftype.upper()+")")
        exit()
    if not (mapping.lower() == "debug" or mapping.lower() == "release"):
        logger.error("mapping of configuration is not correct" +
                     ", it has to be Release or Debug")
        exit()

    if config.has_option('DependencyCollector', 'CREATE_' + conftype.upper()):
        create = config.getboolean('DependencyCollector',
                                   'CREATE_' + conftype.upper())
    else:
        logger.warning("CREATE flag for configratuion not found in config " +
                       "(CREATE_" + conftype.upper() +
                       ") automatically using create method")

    paths_string = config['DependencyCollector']['PATHS_'+mapping.upper()]
    paths = paths_string.split(';')
    for p in paths:
        if not os.path.isdir(p):
            logger.warning("paths in config file: " + p + " does not exist")
    blacklist = []
    if config.has_option('DependencyCollector', 'BLACKLIST'):
        blacklist = config['DependencyCollector']['BLACKLIST']
        blacklist = blacklist.split(';')
    blacklist_lower = []
    for b in blacklist:
        # if line ends with ; an empty string is also in the list
        if b != '':
            blacklist_lower.append(b.lower())
    conf = {'create': create, 'paths': paths, 'blacklist': blacklist_lower}
    return conf


# the update_mode looks up all dll files in the directory of the input file
# and checks if a newer version (according to modified date) of any  dll
# can be found within the paths specified in the config file and copyies the
# libaries into the directory of the input file
def update_mode(infile, conf):
    import glob
    import ntpath
    import time
    import re

    logger.info("running update mode")
    dir = os.path.dirname(os.path.realpath(infile))
    existing_dlls = glob.glob(dir+"/*.dll")

    logger.debug("dll found in directory:" + str(existing_dlls))
    for dll in existing_dlls:
        dllname = ntpath.basename(dll)
        logger.debug("searching for a newer version of " + dll +
                     "("+time.ctime(os.path.getmtime(dll))+")")

        regexp = "(" + ")|(".join(conf['blacklist']) + ")"
        blacklist_match = re.match(
            regexp, dllname.lower())
        if not blacklist_match:
            (newest_dll, mod_date) = search_for_newest_file(dllname,
                                                            conf['paths'])

            if newest_dll == "":
                logger.info("no dll found in given directories for %s"
                            % dllname)
            elif mod_date > os.path.getmtime(dll):
                copy_dll(newest_dll, dir)
            else:
                logger.debug("not copying dll because local file is newer")
        else:
            logger.debug(dll + " skipped because of blacklist")
    return


# create_mode parses recursively the executable resp. library (and their
# dependencies) for dependencies and searches them in the paths specified
# in the config file and copies the newest version (according to modified
# date) into the directory of the input file
def create_mode(infile, conf):
    import ntpath

    logger.info("running create mode")
    path = os.path.dirname(os.path.realpath(infile))
    infile_name = ntpath.basename(infile)
    dlls = search_for_used_dlls(infile_name, path, [], conf)
    logger.debug("all dlls found:" + str(dlls))

    return


# searches recursively all dependencies of the 'infile' and copies
# them into 'path'
def search_for_used_dlls(infile, path, dll_list, conf):
    import re
    dll_regexp = re.compile(b'\.dll')
    logger.debug("analyzing: "+os.path.join(path, infile))
    ifile = open(os.path.join(path, infile), 'rb')
    for line in ifile:
        iterator = dll_regexp.finditer(line)
        for match in iterator:
            pos = match.start()
            while pos > 0 and (line[pos - 1:pos].isalnum() or
                               line[pos - 1:pos] == b"_" or
                               line[pos - 1:pos] == b"-"):
                pos = pos - 1
            if pos == match.start():  # check if position has changed
                continue

            dllname = line[pos:match.end()].decode()

            # MSVC marks delay loaded dlls with the string OCF
            if dllname.startswith("OCF"):
                dllname = dllname.replace("OCF", "", 1)

            regexp = "("+")|(".join(conf['blacklist'])+")"
            blacklist_match = re.match(
                regexp, dllname.lower())
            if not blacklist_match \
               and not dllname.lower() in dll_list:
                (dllpath, mod_date) = \
                    search_for_newest_file(dllname, conf['paths'])
                if dllpath != "":
                    copy_dll(dllpath, path)
                    dll_list.append(dllname.lower())
                    dll_list = \
                        search_for_used_dlls(dllname, path, dll_list, conf)
                else:
                    logger.warning("no file found for %s " % dllname)

    ifile.close()
    logger.debug(infile + " uses dlls:" + str(dll_list))

    return dll_list


# copies the given file 'dllpath' to the 'targetpath'
def copy_dll(dllpath, targetpath):
    import shutil

    try:
        shutil.copy2(dllpath, targetpath)
        logger.info(dllpath + " -> " + targetpath)
    except OSError as error:
        if ("are the same file" in str(error)):
            logger.debug("I don't need to copy " + dllpath)
        else:
            logger.error("unable to copy " + dllpath + " to " +
                         targetpath + "\n(" + str(error) + ")")
    except:
        logger.error("unable to copy " + dllpath + " to " + targetpath)
    return


# searches for the newest 'file' (according to modification date)
# in the given 'paths'
def search_for_newest_file(file, paths):
    import time

    newest_file = ""
    mod_date = ""
    for p in paths:
        fullpath = os.path.join(p, file)
        if os.path.isfile(fullpath) and \
                (mod_date == "" or
                 os.path.getmtime(fullpath) > mod_date):
            mod_date = os.path.getmtime(fullpath)
            newest_file = fullpath
            logger.debug("newest dll found in " + p + " for " + file +
                         " (date:" + time.ctime(mod_date) + ")")

    return(newest_file, mod_date)

if __name__ == "__main__":
    import argparse
    import configparser
    import os
    import time

    start_time = time.time()

    parser = argparse.ArgumentParser(
       description='searches for dependencies of an executable or library and'
                   'copies the dependencies to the corresponding directory')

    parser.add_argument('--infile', default='', metavar="inputfile",
                        help="""executable or dependency which dependencies
                        should be copied""",
                        required=True)
    parser.add_argument('--configfile', default='config.ini',
                        metavar="configfile",
                        help="""configuration file of the
                        dependencycollector""", required=True)
    parser.add_argument('--configuration', default='Release',
                        metavar='configuration',
                        help="""current build configuration
                        (Release|Debug|...)""", required=True)
    parser.add_argument('--debug', action="store_true",
                        help="""enable debug messages""")
    parser.add_argument('--create', action="store_true",
                        help="""force create mode""")

    args = parser.parse_args()

    if args.debug:
        logger.setLevel(logging.DEBUG)

    if args.create:
        logger.info("create mode forced due to argument")

    if not os.path.isfile(args.infile):
        logger.error("input file does not exist: " + args.infile)
        exit()
    if not os.path.isfile(args.configfile):
        logger.error("config file does not exist: " + args.configfile)
        exit()

    logger.debug("processing: " + args.infile)
    logger.debug("config: " + args.configuration)
    conf = parse_config_file(args.configfile, args.configuration)
    conf['localpath'] = os.path.dirname(os.path.realpath(args.infile))

    # add local path - there might be our own-built dependencies
    conf['paths'].append(os.path.dirname(
                         os.path.realpath(args.infile)))  # adding local path

    logger.debug("running create mode: " + str(conf['create']))
    logger.debug("using paths: " + str(conf['paths']))
    logger.debug("using blacklist: " + str(conf['blacklist']))

    # add my name to blacklist - we don't need to copy it
    conf['blacklist'].append(os.path.basename(args.infile).lower())

    if conf['create'] is True or args.create:
        create_mode(args.infile, conf)
        logger.debug("resetting create flag")
        config = configparser.ConfigParser()
        config.read(args.configfile)
        config['DependencyCollector']['CREATE_' +
                                      args.configuration.upper()] = "False"
        with open(args.configfile, 'w') as configfile:
            config.write(configfile)

    elif conf['create'] is False:
        update_mode(args.infile, conf)
    else:
        logger.error("create mode unkown")
        exit()

    logger.info("finished in %s seconds" %
                round((time.time() - start_time), 2))



#!/usr/bin/env python
"""Increments the build version of a C++ projects.

This script increments the build version after every build of 
a Visual Stuido project. It manipulates the DkVersion.h file in order
to do so. The version number defined there is displayed in help and
used for the VS_VERSION_INFO in the *.rc file
"""
import logging
import os

__author__ = "Markus Diem"
__credits__ = ["Markus Diem"]
__license__ = "GPLv3"
__version__ = "0.2"
__maintainer__ = "Markus Diem"
__email__ = "markus@nomacs.org"
__status__ = "Production"


OUTPUT_NAME = "versionincrement"

def increment(line: str, newversion: str):

    str_ver = line.split(":")

    # searching version.cache: version:0.3.5
    if "version" in line and len(str_ver) == 2:

        old_v = str_ver[1].split(".")

        if len(old_v) == 3:
            # increment
            build = str(int(old_v[-1])+1)

        line = str_ver[0] + ":" + newversion + "." + build

        # send status message only once
        print("[Version Incrementer] " + line.replace("version:", "version updated: "))

    return line


def update_version(version: str):
    from shutil import move
    from os import remove

    from utils.fun import version_cache

    filepath = version_cache()
    tmppath = filepath + "tmp"

    if os.path.exists(filepath):
        with open(filepath, "r") as src:
            with open(tmppath, "w") as dst:
                for l in src.readlines():

                    l = increment(l, version)
                    dst.write(l)
    
        # swap
        remove(filepath)
        move(tmppath, filepath)

    else:
        with open(filepath, "w") as dst:
            dst.write("version:" + version)


if __name__ == "__main__":
    import argparse

    parser = argparse.ArgumentParser(
       description='Increments the version in a version cache file.')

    parser.add_argument("version", type=str,
                        help="""current nomacs version - typically optained from cmake""")

    args = parser.parse_args()

    update_version(args.version)



#!/usr/bin/env python
"""Increments the build version of a C++ projects.

This script increments the build version after every build of
a Visual Stuido project. It manipulates the DkVersion.h file in order
to do so. The version number defined there is displayed in help and
used for the VS_VERSION_INFO in the *.rc file
"""

__author__ = "Markus Diem"
__credits__ = ["Markus Diem"]
__license__ = "GPLv3"
__version__ = "0.3"
__maintainer__ = "Markus Diem"
__email__ = "markus@nomacs.org"
__status__ = "Production"


OUTPUT_NAME = "versionupdate"


def update(filepath, copy=False):
    from shutil import move
    from os import remove
    from utils.fun import version

    v = version()

    dstpath, ext = os.path.splitext(filepath)
    dstpath += "-versioned" + ext

    with open(filepath, "r") as src:
        with open(dstpath, "w") as dst:
            for l in src.readlines():
                l = update_version_string(v, l)
                l = update_version_rc(v, l)
                l = update_version_patch(v, l)

                dst.write(l)

    # replace current file?
    if not copy:
        remove(filepath)
        move(dstpath, filepath)


def update_version_string(version, line):
    # searching (DkVersion.h): #define NOMACS_VERSION_STR "3.14.42\0"
    # searching (msi installer): <?define ProductVersion = "3.14.42"?>
    # searching (inno installer): define MyAppVersion "3.14.42"
    if (
        "NOMACS_VERSION_STR" in line
        or "<?define ProductVersion" in line
        or "define MyAppVersion" in line
    ):
        str_ver = line.split('"')
        line = str_ver[0] + '"' + version + '"' + str_ver[-1]

    return line


def update_version_rc(version, line):
    # searching: #define NOMACS_VERSION_RC 3,14,42
    if "NOMACS_VERSION_RC" in line:
        str_ver = line.split(" ")
        str_ver[-1] = version.replace(".", ",")

        line = " ".join(str_ver) + "\n"

    return line


def update_version_patch(version, line):
    # get patch from 3.14.42
    vs = version.split(".")

    if len(vs) != 3:
        print("WARNING: could not split version: " + version)
        return

    # searching: #define NOMACS_VER_PATCH 0
    if "NOMACS_VER_PATCH" in line:
        str_ver = line.split(" ")
        str_ver[-1] = vs[-1]

        line = " ".join(str_ver) + "\n"

    return line


if __name__ == "__main__":
    import argparse
    import os

    parser = argparse.ArgumentParser(
        description="Increments the build version of a C++ project."
    )

    parser.add_argument(
        "inputfile",
        type=str,
        help="""full path to the file who's version should be updated""",
    )
    parser.add_argument(
        "--copy", action="store_true", help="""if set, a _ver file will be created"""
    )

    args = parser.parse_args()

    if not os.path.isfile(args.inputfile):
        print("input file does not exist: " + args.inputfile)
        exit()

    update(args.inputfile, args.copy)




def pack(base: str):
    import os
    import shutil
    from utils.fun import version

    version = version()

    pname = "nomacs-" + version
    binarydir = "/installer/nomacs.x64"
    dstdir = "/installer/" + pname
    licenses = "/ImageLounge/license"
    
    zipname = "nomacs-portable-win.zip"
    zippath = base + "/installer/" + zipname
    dst = base + dstdir

    # collect all files
    copytree(base + binarydir, dst)
    copytree(base + licenses, dst + "/license")

    # create empty settings.ini (nomacs will then recognize that it's portable)
    open(dst + "/settings.ini", 'w').close()

    # zip
    nf = zip(dst, zippath, pname)

    # clean up
    shutil.rmtree(dst)

    zipsize = os.stat(zippath).st_size / (1024*1024) # -> MB
    print(("[nomacs portable] %d files zipped to %s [%.2f MB] ") % (nf, zipname, zipsize))

def zip(src: str, dst: str, zipbase: str):
    import zipfile

    numfiles = 0

    with zipfile.ZipFile(dst, 'w', zipfile.ZIP_DEFLATED) as zh:

        # zip everything
        for root, dirs, files in os.walk(src):
            base = root.replace(src, zipbase)

            for file in files:
                zh.write(os.path.join(root, file), os.path.join(base, file))
                numfiles += 1

    return numfiles

def copytree(src: str, dst: str, ext: str = "", symlinks: bool = False, ignore: bool = None):
    import shutil
    import os

    if not os.path.exists(dst):
        os.mkdir(dst)

    for item in os.listdir(src):
        s = os.path.join(src, item)
        d = os.path.join(dst, item)

        if os.path.isdir(s):
            copytree(s, d, ext, symlinks, ignore)
        elif ext:
            if s.endswith(ext):
                shutil.copyfile(s, d)
        else:
            shutil.copyfile(s, d)


if __name__ == "__main__":
    import argparse
    import sys, os
    from utils.fun import repopath

    parser = argparse.ArgumentParser(
        description='packs nomacs portable.')

    rp = repopath(sys.argv[0])

    args = parser.parse_args()

    if not os.path.exists(rp):
        print("repository path does not exist: " + rp)
        exit()

    pack(rp)



# Change following line to force rebuild of AppVeyor cache:
# Last rebuild: 2025-05-09

import os
from utils.config import Config

class NomacsConfig(Config):

    def __init__(self, params):
        super().__init__(params, "nomacs")

    def defaults(self):

        if not self.libpath:
            self.libpath = self.repopath + "/3rd-party/build"

        if "srcpath" not in self.__dict__ or not self.srcpath:
            self.srcpath = self.repopath + "/ImageLounge"

        # this is not called 'path' because it won't exist until we call cmake
        if "builddir" not in self.__dict__ or not self.builddir:
            self.builddir = self.repopath + "/build"

        super().defaults()

    def cmake_args(self):

        # tune cmake parameters here
        args = [
            "-DQT_VERSION_MAJOR=6",
            "-DCMAKE_PREFIX_PATH=" + self.qtpath,
            "-DDEPENDENCY_PATH=" + self.libpath,
            "-DENABLE_QUAZIP=ON",
            "-DENABLE_TRANSLATIONS=ON",
            "-DENABLE_HEIF=ON",
            "-DENABLE_AVIF=ON",
            "-DENABLE_JXL=ON",
            "-DENABLE_JXR=ON",
            "-DENABLE_INCREMENTER=OFF",
            "-B" + self.builddir,
            self.srcpath
        ]

        return args


class ExpatConfig(Config):

    def __init__(self, params):
        super().__init__(params, "expat")

        self.install = False

    def defaults(self):

        # this is not called 'path' because it won't exist until we call cmake
        if "builddir" not in self.__dict__ or not self.builddir:
            self.builddir = os.path.join(self.repopath, "build", self.name)

        super().defaults()

        self.binaryfile = os.path.join(self.builddir, "Release", "libexpat.dll")

    def cmake_args(self):

        # tune cmake parameters here
        args = [
            "-DBUILD_examples=OFF",
            "-DBUILD_tests=OFF",
            "-B" + self.builddir,
            self.srcpath + "/expat"
        ]

        return args


class ExifConfig(Config):

    def __init__(self, params):
        super().__init__(params, "exiv2")

        self.install = False

    def defaults(self):

        if "libpath" not in self.__dict__ or not self.libpath:
            self.libpath = os.path.join(self.repopath, "build")

        # this is not called 'path' because it won't exist until we call cmake
        if "builddir" not in self.__dict__ or not self.builddir:
            self.builddir = os.path.join(self.repopath, "build", self.name)

        super().defaults()

        self.binaryfile = os.path.join(
            self.builddir, "Release", "bin", "exiv2.dll")

    def cmake_args(self):

        # tune cmake parameters here
        args = self.additional_cmake_args + [
            "-DEXPAT_BUILD_PATH=" + self.libpath + "/expat",
            "-DEXPAT_INCLUDE_DIR=" + self.repopath + "/expat/expat/lib",
            "-DEXIV2_ENABLE_BMFF=ON",
            "-DEXIV2_ENABLE_BROTLI=OFF",
            "-DEXIV2_ENABLE_INIH=OFF",
            "-DEXIV2_BUILD_EXIV2_COMMAND=OFF",
            "-DEXIV2_BUILD_SAMPLES=OFF",
            "-B" + self.builddir,
            self.srcpath
        ]

        return args


class LibrawConfig(Config):

    def __init__(self, params):
        super().__init__(params, "libraw")

        self.install = False

    def defaults(self):

        # this is not called 'path' because it won't exist until we call cmake
        if "builddir" not in self.__dict__ or not self.builddir:
            self.builddir = os.path.join(self.repopath, "build", self.name)

        super().defaults()

        self.binaryfile = os.path.join(self.builddir, "Release", "raw.dll")

    def cmake_args(self):

        # tune cmake parameters here
        args = self.additional_cmake_args + [
            "-DENABLE_EXAMPLES=OFF",
            "-B" + self.builddir,
            self.srcpath
        ]

        return args


class QuazipConfig(Config):

    def __init__(self, params):
        super().__init__(params, "quazip")

        self.install = False

    def defaults(self):

        # this is not called 'path' because it won't exist until we call cmake
        if "builddir" not in self.__dict__ or not self.builddir:
            self.builddir = os.path.join(self.repopath, "build", self.name)

        if "libpath" not in self.__dict__ or not self.libpath:
            self.libpath = os.path.join(self.repopath, "build")

        super().defaults()

        self.binaryfile = os.path.join(self.builddir, "quazip", "Release", "quazip1-qt6.dll")

    def cmake_args(self):

        # tune cmake parameters here
        args = self.additional_cmake_args + [
            "-DCMAKE_PREFIX_PATH=" + self.qtpath,
            "-B" + self.builddir,
            "-DQUAZIP_BZIP2=OFF",
            "-DQUAZIP_BZIP2_STDIO=OFF",
            "-DQUAZIP_QT_MAJOR_VERSION=6",
            self.srcpath
        ]

        return args

    def cmake_build_args(self):

        # only build shared lib
        args = [
            "--target QuaZip"
        ]

        return args

class OpenCVConfig(Config):

    def __init__(self, params):
        super().__init__(params, "opencv")

        self.install = False

    def defaults(self):

        if "srcpath" not in self.__dict__ or not self.srcpath:
            self.srcpath = os.path.join(self.repopath, self.name)

        # this is not called 'path' because it won't exist until we call cmake
        if "builddir" not in self.__dict__ or not self.builddir:
            self.builddir = os.path.join(self.repopath, "build", self.name)

        super().defaults()

        # FIXME: release is generated by cmake - so this is not enough to ask for...
        self.binaryfile = os.path.join(
            self.builddir, "bin", "Release", "opencv_core4110.dll")

    def cmake_zlib(self):

        args = [
            "-DZLIB_INCLUDE_DIR=" + self.srcpath + "/3rdparty/zlib",
            "-DZLIB_BUILD_PATH=" + self.builddir + "/3rdparty",
            "-DZLIB_LIBRARY_RELEASE=" + self.builddir + "/3rdparty/lib/Release/zlib.lib",
        ]

        return args

    def cmake_args(self):

        # tune cmake parameters here
        args = [
            "-DBUILD_PERF_TESTS=OFF",
            "-DBUILD_TESTS=OFF",
            "-DBUILD_opencv_java=OFF",
            "-DBUILD_opencv_java_bindings_generator=OFF",
            "-DBUILD_opencv_python=OFF",
            "-DBUILD_opencv_apps=OFF",
            "-DBUILD_opencv_dnn=OFF",
            "-DBUILD_opencv_calib3d=OFF",
            "-DBUILD_opencv_highgui=OFF",
            "-DBUILD_opencv_photo=OFF",
            "-DBUILD_opencv_python2=OFF",
            "-DBUILD_opencv_python3=OFF",
            "-DBUILD_opencv_python_tests=OFF",
            "-DBUILD_opencv_python_bindings_generator=OFF",
            "-DBUILD_opencv_stitiching=OFF",
            "-DBUILD_opencv_video=OFF",
            "-DBUILD_opencv_videoio=OFF",
            "-B" + self.builddir,
            self.srcpath
        ]

        return args


class FormatsConfig(Config):

    def __init__(self, params, name: str):
        super().__init__(params, name)

    def defaults(self):

        if "libpath" not in self.__dict__ or not self.libpath:
            self.libpath = os.path.join(self.repopath, "build")

        # only build release for imageformats        
        self.buildconfig = "release"

        super().defaults()



    def cmake_args(self):

        # tune cmake parameters here
        args = [
            "-DQT_VERSION_MAJOR=6",
            "-DCMAKE_PREFIX_PATH=" +
            # os.path.join(self.libpath, "libde265") + ";" +
            # os.path.join(self.libpath, "libheif") + ";" +
            self.qtpath,
            "-DENABLE_QOI=ON",
            "-DENABLE_WEBP=OFF",
            "-DENABLE_DDS=ON",
            "-B" + self.builddir,
            self.srcpath
        ]

        return args


def make_libs(params):
    from utils.build import build

    params['install'] = False

    opencv = OpenCVConfig(params)
    # some script debugging options:
    # opencv.force = True
    print(opencv)
    build(opencv)

    # Exiv2
    expat = ExpatConfig(params)
    build(expat)

    exiv2 = ExifConfig(params)
    exiv2.additional_cmake_args = opencv.cmake_zlib()
    build(exiv2)

    # libraw
    libraw = LibrawConfig(params)
    build(libraw)

    # quazip
    quazip = QuazipConfig(params)
    quazip.additional_cmake_args = opencv.cmake_zlib()
    build(quazip)

    # uncomment for debugging
    # print(expat)

def make_imageformats(params):
    
    params['install'] = False

    # config libde265 which we need for libheif
    # libde265 = FormatsConfig(params, "libde265")
    # libde265.builddir = os.path.join(libde265.builddir, libde265.name)
    # libde265.binaryfile = os.path.join(libde265.builddir, libde265.name, "Release", libde265.name + ".dll")
    # build(libde265)
    
    # config libheif
    # libheif = FormatsConfig(params, "libheif")
    # libheif.builddir = os.path.join(libheif.builddir, libheif.name)
    # libheif.binaryfile = os.path.join(libheif.builddir, "libheif", "Release", "heif.dll")
    # build(libheif)
    
    # configure image formats
    params["srcpath"] = params["repopath"]
    params['install'] = True
    imageformats = FormatsConfig(params, "imageformats")
    build(imageformats)

    # uncomment for debugging
    # print(libde265)
    # print(libheif)
    # print(imageformats)

def configure_libs(p, config):
    # from makelibs import make as ml
    # from makeimageformats import make as mi

    p['repopath'] = config.repopath + "/3rd-party"

    make_libs(p)

    p['libpath']  = config.libpath + "/imageformats"
    p['repopath'] = config.repopath + "/3rd-party/imageformats"
    p['builddir'] = config.repopath + "/3rd-party/build/imageformats"

    make_imageformats(p)


if __name__ == "__main__":
    import argparse
    import sys
    
    from utils.fun import repopath
    from utils.build import build

    parser = argparse.ArgumentParser(
        description='packs nomacs portable.')

    parser.add_argument("qtpath", type=str,
                        help="""path to your Qt folder""")
    parser.add_argument('--lib-path', dest='libpath', type=str, default="",
                        help='additional cmake arguments')
    parser.add_argument('--repo-path', dest='repopath', type=str, default="",
                        help='path to the nomacs repository')
    parser.add_argument('--build-dir', dest='builddir', type=str, default="",
                        help='Specify the build directory')
    parser.add_argument('--build-config', dest='buildconfig', type=str, default="",
                        help='build configuration [debug|release]')
    parser.add_argument('--project', dest='project', type=str, default="all",
                        help='comma separated name of the project(s) to be built (i.e. project=opencv,expat)')
    parser.add_argument('--force', action='store_true',
                    help='forces building the project')
    parser.add_argument('--configure', action='store_true',
                        help='if set, projects are only configured rather than built')


    # make args a dict
    params = vars(parser.parse_args())

    # get the repository path
    if not params['repopath']:
        params['repopath'] = repopath(sys.argv[0])

    params['project'] = params['project'].split(',')

    c = NomacsConfig(params)

    if not params['libpath']:
        configure_libs(params, c)

    # uncomment for debugging
    print(c)

    build(c)



import os

class Config(object):

    def __init__(self, params, name: str):
        self.__dict__.update(params)
        self.name = name

        self.additional_cmake_args = []

        self.defaults()
        self.check()

    def check(self):

        # check if all directories exist
        for key in self.__dict__:

            p = self.__dict__[key]

            if p:
                if key.endswith("path"):
                    if not os.path.exists(p):
                        print("[WARNING] %s path does not exist: %s" % (key, p))
                    else:
                        self.__dict__[key] = Config.normpath(p)

        if "builddir" in self.__dict__:
            self.builddir = Config.normpath(self.builddir)

    @staticmethod
    def normpath(path: str):
        tmp = os.path.abspath(path)
        # cmake has issues with windows path separators
        tmp = tmp.replace("\\", "/")
        return tmp

    def defaults(self):

        # "debug" builds debug and release
        if "buildconfig" not in self.__dict__ or not self.buildconfig:
            self.buildconfig = "debug"

        # call install?
        if "install" not in self.__dict__:
            self.install = True

        # if set to True, we'll run a dry run (only configure cmake)
        if "cmakeonly" not in self.__dict__:
            self.cmakeonly = False

        # if set, we can skip builds
        if "binaryfile" not in self.__dict__:
            self.binaryfile = ""

        # if true, we will always rebuild
        if "force" not in self.__dict__:
            self.force = False

        if "srcpath" not in self.__dict__ or not self.srcpath:
            self.srcpath = os.path.join(self.repopath, self.name)

        # this is not called 'path' because it won't exist until we call cmake
        if "builddir" not in self.__dict__ or not self.builddir:
            self.builddir = os.path.join(self.repopath, "build")
        else:
            self.builddir = os.path.abspath(self.builddir)

    def cmake_args(self):

        return ""

    def cmake_build_args(self):
    
        return ""

    def __str__(self):

        keyvals = [key + ": " + str(self.__dict__[key])
                   for key in self.__dict__]
        return "\n".join(keyvals)

    def is_built(self):

        if self.force:
            return False

        return os.path.isfile(self.binaryfile)
